/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/05/2011: 	Modified.						*/
/* 04/24/2011: 	Previouly modified.					*/
/* 04/22/2011: 	Created.						*/
/************************************************************************/

/************************************************************************/
/* Program: 	bfs_betweenness().					*/
/* Function: 	Betweenness centrality based on BFS SSSP algorithm.	*/					*/
/************************************************************************/
mata:
real vector bfs_betweenness(real matrix E, 
  | string scalar nodots){
  /* Declare variables.							*/
  struct matrix_struct scalar z /* Return structure.			*/
  real matrix D /* Distance matrix.					*/
  real matrix W /* Weight matrix.					*/
  real matrix L /* Leaf vertex matrix.					*/
  real matrix A /* Adjacency list.					*/
  real matrix B /* Betweenness score matrix.				*/
  real vector n1 /* Extended neighbor(s) at dist. d.			*/
  real vector n2 /* Ext-ext neighbors at d+1 from source vertex.	*/
  real vector n3 /* Nbhrs at d+1 from source and nbhrs of n1.		*/  
  real vector b /* Betweenness centrality return vector.		*/
  real vector a /* Adjacency list vector.				*/
  real scalar m /* Total number of nodes.				*/
  real scalar s /* Source vertex.					*/
  real scalar j /* Column counter.					*/
  real scalar k /* Column counter.					*/  
  real scalar d /* Distance from source vertex.				*/
  real scalar l1 /* Leaf score for node n1[j].				*/
  real scalar l2 /* Leaf score for node n2[k].				*/
  real scalar w1 /* Weight score for node n1[j].			*/
  real scalar w2 /* Leaf score for node n2[k].				*/  
  string scalar o /* Output string.					*/
  /**********************************************************************/
  /* Begin program.							*/
  /**********************************************************************/
  /***** Run BFS algorithm and obtain matrices.				*/
  z=bfs_sp(E,nodots); D=z.X1; W=z.X2; L=z.X3; A=z.X4
  /* Obtain total number of nodes in network.				*/
  m=rows(A)
  /***** Inform user of program starting.				*/
  o="noisily display "+`""""'; stata(o); displayflush()
  o="Betweenness centrality calculation ("+strofreal(m)+" vertices)"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()
  /* Display dots unless specified not to.				*/
  if(nodots==""){
    o="----+--- 1 ---+--- 2 ---+--- 3 ---+--- 4 ---+--- 5"
    o="noisily display "+`"""'+o+`"""'
    stata(o); displayflush() 
  }
  /**********************************************************************/
  /* Loop through source vertices.					*/
  /**********************************************************************/
  for(s=1;s<=m;s++){
    /* Obtain max distance from source vertex.				*/
    d=max(D[s,.])-1
    /* Loop until distance reaches zero.  Move from bottom up.		*/
    while(d>-1){
      /* Find extended neighbors at distance d from source vertex.	*/
      n1=select((1..m),D[s,.]:==d)
      /* Loop through extended neighbors that were just discovered.	*/
      for(j=1;j<=cols(n1);j++){
        /* Find nodes that lie at distance d+1 from source vertex.	*/
        n2=select((1..m),D[s,.]:==d+1)
        /* Find nodes that are nbhrs of n1[1,j] & at d+1 distance.	*/
        a=A[n1[j],.]; n3=J(1,0,.)
        for(k=1;k<=nonmissing(a);k++){
          if(anyof(n2,a[k])==1) n3=(n3,a[k])
        }
        /* Set initial betweenness score to one.			*/
        L[s,n1[j]]=1
        /* Loop through extended-extended neighbors.			*/
        for(k=1;k<=cols(n3);k++){
          /* Assign temporary leaf and weight scores.			*/
          l1=L[s,n1[j]]
          l2=L[s,n3[k]]
          w1=W[s,n1[j]]
          w2=W[s,n3[k]]  
          /* Re-assign leaf score if all values present.		*/
	  if(l2!=.&w1!=.&w2!=.&w2!=0) L[s,n1[j]]=l1+l2*w1/w2
        }
      }
      /* De-crement distance.						*/
      --d
    }
    /* Display status unless specified not to.				*/
    if(nodots=="") display_status(m,s)
  }
  /* Generate betweenness centrality vector.				*/
  B=L:-1
  _diag(B,0)
  b=colsum(B)'
  /* Notify program is over.						*/
  o="Betweenness centrality calculation completed"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()  
  /* Return betweenness centrality vector.				*/
  return(b)
}  
end


  