/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/01/2011: 	Modified.						*/
/* 04/30/2011: 	Previouly modified.					*/
/* 04/14/2011: 	Created.						*/
/* Program: 	Class wrapper for Stata Graph Library (SGL) functions.	*/
/************************************************************************/
/* Set Stata version.							*/
version 11.1	
/* Initiate Mata.							*/
mata:
/* Set Mata system parameters.						*/
mata set matastrict on /* Force variable declaration on/off.		*/
mata set matalnum off /* Turn program line-number tracing on/off.	*/
/************************************************************************/
/* Define class.							*/
/************************************************************************/
class network_measure{
  /* CLASS MEMBER VARIABLE: RESULTS MATRIX.				*/
  real matrix res  
  /* CLASS MEMBER VARIABLE: EDGE LIST (|E|x3).				*/
  real matrix E
  /* CLASS MEMBER VARIABLE: SUPPRESS STATUS DOTS OPTION.		*/
  string scalar nodots
  /* CLASS MEMBER VARIABLE: INFINITE DISTANCE REPLACEMENT.		*/
  real scalar infinity
  /* CLASS MEMBER VARIABLE: STRING INDICATOR TO USE POWER ITERATION.	*/
  string scalar power
  /* CLASS MEMBER VARIABLE: KATZ-BONACICH CENTRALITY BETA VECTOR.	*/
  real vector beta		
  /* CLASS MEMBER VARIABLE: RIGHT EIGENVECTOR INDICATOR.		*/
  string scalar notranspose
  /* CLASS MEMBER VARIABLE: ALPHA PARAMETER.				*/
  real scalar alpha
  /* CLASS CONSTRUCTOR.							*/
  void new()
  /* GENERATE ADJACENCY MATRIX.						*/
  real scalar adjacency()
  /* GENERATE DISTANCE MATRIX.						*/
  real scalar distance()
  /* GENERATE PATH MATRIX.						*/
  real scalar path()
  /* GENERATE BETWEENNESS CENTRALITY.					*/
  real scalar betweenness()
  /* GENERATE OVERALL/AVERAGE & LOCAL CLUSTERING COEFFICIENTS.		*/
  real scalar clustering()
  /* GENERATE EIGENVECTOR CENTRALITY.					*/
  real scalar eigenvector()
  /* GENERATE INVERSE OF DOMINANT EIGENVALUE.				*/
  real scalar maxalpha()
  /* GENERATE KATZ-BONACICH CENTRALITY.					*/
  real scalar katzbonacich()
}
/************************************************************************/
/* Define constructor.							*/
/************************************************************************/
void network_measure::new(){
  nodots=""
  infinity=-101
  power=""
  notranspose=""
  alpha=1
}
/************************************************************************/
/* Define structures.							*/
/************************************************************************/
struct matrix_struct{
  real matrix X1,X2,X3,X4,X5,X6
}
/************************************************************************/
/************************************************************************/
/* Program: adjacency().						*/
/* Function: Generate adjacency matrix.					*/
/************************************************************************/
real scalar network_measure::adjacency(){
  /* Convert sparse matrix (edge list) to full matrix.			*/
  res=::sparse2full(E,0) 
  /* Store values in e().						*/
  ::ereturn(res)
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: distance().							*/
/* Function: Generate distance matrix from BFS or Dijkstra's algorithm.	*/
/************************************************************************/
real scalar network_measure::distance(){
  /* Declare variables. 						*/
  struct matrix_struct scalar z /* Matrix return structure.		*/
  /* Implement breadth-first search if all weights equal to one.	*/
  if(allof(E[.,3],1)) z=::bfs_sp(E,nodots)
  /* Implement Dijkstra's algorithm if weights unequal.			*/
  else z=::dijkstra_sp(E,nodots)
  /* Store values in e().						*/
  ::ereturn(z.X1)
  /***** Replace inifinite distances if specified or left empty.	*/
  if(infinity==-101) res=z.X1
  else res=editmissing(z.X1,infinity)
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: path().							*/
/* Function: Generate path matrix from BFS or Dijkstra's algorithm.	*/
/************************************************************************/
real scalar network_measure::path(){
  /* Declare variables. 						*/
  struct matrix_struct scalar z /* Matrix return structure.		*/
  /* Implement breadth-first search if all weights equal to one.	*/
  if(allof(E[.,3],1)) z=::bfs_sp(E,nodots) 
  /* Implement Dijkstra's algorithm if weights unequal.			*/
  else z=::dijkstra_sp(E,nodots)
  /* Set to zero if path from node i to j does not exist.		*/
  _editmissing(z.X2,0)
  /* Store values in e().						*/
  ::ereturn(z.X2)
  /* Map to results matrix. 						*/
  res=z.X2
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: betweenness().						*/
/* Function: Generate vector of betweenness centrality scores.		*/
/************************************************************************/
real scalar network_measure::betweenness(){
  /* Implement breadth-first search if all weights equal to one.	*/
  if(allof(E[.,3],1)) res=::bfs_betweenness(E,nodots)
  /* Implement Dijkstra's algorithm if weights unequal.			*/
  else res=::dijkstra_betweenness(E,nodots)
  /* Store values in e().						*/
  ::ereturn(res)
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: clustering().						*/
/* Function: Generate overall/average and local clustering coefficients.*/
/************************************************************************/
real scalar network_measure::clustering(){
  /* Declare variables. 						*/
  struct matrix_struct scalar z /* Matrix return structure.		*/  
  string matrix rs /* Row names.					*/
  string matrix cs /* Column names.					*/
  real matrix C /* Overall/average clustering coefficients.		*/
  /* Implement clustering coefficient calculation.			*/
  z=::clustering_coefficients(E,nodots)
  C=z.X1; res=z.X2
  /* Store overall/average clustering coefficients in e().		*/
  st_matrix("e(cc)",C)
  /* Apply matrix labels to clustering coefficients vector.		*/	
  rs=("","Overall"\"","Average")						
  cs=("","Arithmetic_mean"\"","Geometric_mean"\"","Maximum"\"","Minimum")					
  st_matrixrowstripe("e(cc)",rs)						
  st_matrixcolstripe("e(cc)",cs)     
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: eigenvector().						*/
/* Function: Generate eigenvector centrality.				*/
/* Note: Error detected in eigensystemselecti(A',(1,1),e,l).		*/
/************************************************************************/
real scalar network_measure::eigenvector(){
  /* Declare variables. 						*/
  struct matrix_struct scalar z /* Matrix return structure.		*/
  real matrix A /* Adjacency matrix.					*/  
  real vector l /* Eigenvalue(s).					*/
  complex matrix X /* Eigenvector matrix.				*/
  complex vector x /* Eigenvalue vector.				*/
  string scalar o /* User inform string.				*/
  /***** Use Mata's routine if 'power' option not specified.		*/
  if(power==""){
    /* Obtain adjacency matrix.						*/
    A=::sparse2full(E,0)  
    /***** Implement symeigenselecti if matrix symmetric.		*/
    if(issymmetric(A)==1){
      _symeigensystem(A,X,x); res=abs(Re(X[.,1])); l=abs(Re(x[1]))
    }
    /***** Implement eigensystemselecti if matrix asymmetric.		*/
    /* Default: Transpose of left eigenvector.				*/
    else if(issymmetric(A)==0 & notranspose==""){
      /* Transpose A and use regular routine for right eigenvector.	*/
      /* Left eigenvector from LAPACK routine seems to be off.		*/
      /* Refer to rhdtlapack's post at 					*/
      /* http://icl.cs.utk.edu/lapack-forum/viewtopic.php?f=2&t=2119	*/
      /* 'Problem with left eigenvectors returned by ZGEEV'		*/
      /* posted Wed Dec 29, 2010 12:41 pm.				*/
      _eigensystem(A',X,x); res=abs(Re(X[.,1])); l=abs(Re(x[1]))
    }
    /* Calculate right eigenvector if requested by user.		*/
    else{
      _eigensystem(A,X,x); res=abs(Re(X[.,1])); l=abs(Re(x[1]))
      o="right eigenvector requested"
      o="noisily display "+`"""'+o+`"""'
      stata(o); displayflush()      
    }
  }
  /***** Implement power iteration method if specified.			*/
  /* No need for separate routine for symmetric matrix since left and	*/
  /* right eigenvectors the same. 					*/
  else{
    /* Default: Calculate left eigenvector.				*/
    if(notranspose==""){
      z=::power(E,nodots,"left")
      /***** Mata error 3360 - convergence not achieved.		*/
      if(z.X1==3360) return(3360)
      else{
	res=z.X1; l=z.X2
      }
    }
    /* Calculate right eigenvector if requested by user.		*/
    else{
      o="right eigenvector requested"
      o="noisily display "+`"""'+o+`"""'
      stata(o); displayflush()	  
      z=::power(E,nodots)
      /***** Mata error 3360 - convergence not achieved.		*/
      if(z.X1==3360) return(3360)
      else{
	res=z.X1; l=z.X2  
      }      
    }   
  }
  /* Return dominant eigenvalue in e().					*/
  st_numscalar("e(l)",l)  
  /* Store values in e().						*/
  ::ereturn(res)
  /* Success.								*/
  return(0)
}  
/************************************************************************/
/* Program: maxalpha().							*/
/* Function: Generate inverse of dominant eigenvalue.			*/
/************************************************************************/
real scalar network_measure::maxalpha(){
  /* Declare variables. 						*/
  struct matrix_struct scalar z /* Matrix return structure.		*/
  real matrix A /* Adjacency matrix.					*/
  /***** Use Mata's routine if 'power' option not specified.		*/
  if(power==""){
    /* Obtain adjacency matrix.						*/
    A=::sparse2full(E,0)  
    /* Obtain eigenvalues.						*/
    res=_eigenvalues(A); res=abs(Re(res[1]))
  }
  else{
    z=::power(E,nodots)
    /***** Mata error 3360 - convergence not achieved.			*/
    if(z.X1==3360) return(3360)
    else res=z.X2
  }
  /***** Return inverse of dominant eigenvalue.				*/
  st_numscalar("e(alpha_max)",1/res)
  /* Obtain inverse.							*/
  res=1/res
  /* Success.								*/
  return(0)
}
/************************************************************************/
/* Program: katzbonacich().						*/
/* Function: Generate Katz-Bonacich centrality.				*/
/* NOTE: Network CANNOT start with isolates and/or loops if user is	*/
/* specifying his/her own beta vector in Mata.				*/
/************************************************************************/
real scalar network_measure::katzbonacich(){
  /* Declare variables.							*/
  real matrix b /* Beta vector.						*/
  real matrix A /* Adjacency matrix.					*/
  real scalar m /* Number of nodes in network.				*/ 
  /* Obtain node count.							*/
  m=max(E[.,(1,2)])
  /***** Specify beta vector of exogenous factors.			*/
  /* If no beta vector specified, default to vector of all ones.	*/
  if(beta==J(1,0,.)) b=J(m,1,1)
  /* Use specified beta vector if provided.				*/
  else b=beta
  /***** Calculate Katz-Bonacich centrality.				*/
  /***** Implement power iteration alogrithm if specified.		*/
  if(power!=""){
    if(notranspose==""){
      res=::power_katzbonacich(E,alpha,b,nodots,"left")
      /***** Mata error 3360 - convergence not achieved.		*/
      if(res==3360) return(3360)
    }
    else{
      res=::power_katzbonacich(E,alpha,b,nodots,"")
      /***** Mata error 3360 - convergence not achieved.		*/
      if(res==3360) return(3360)
    }    
  }
  /***** Implement Mata routine if 'power' not specified.		*/
  else{
    A=::sparse2full(E,0)
    if(issymmetric(A)==1){
      res=invsym(I(rows(A))-alpha*A)*b
    }
    else if(issymmetric(A)==0 & notranspose==""){
      A=A'; res=qrinv(I(rows(A))-alpha*A)*b
    }
    else res=qrinv(I(rows(A))-alpha*A)*b
  }
  /* Store values in e().						*/
  st_numscalar("e(alpha)",alpha); ::ereturn(res)      
  /* Success.								*/
  return(0)
}
end


  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  