/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/02/2011: 	Modified.						*/
/* 04/30/2011: 	Previouly modified.					*/
/* 04/14/2011: 	Created.						*/
/* Program: 	Wrapper to generate network measure.			*/
/************************************************************************/
/* Declare program and Stata version.					*/
program network, sortpreserve eclass
version 11.1
/************************************************************************/
/****** 		DEFINE SYNTAX				    *****/
/************************************************************************/
#d;
  syntax varlist(min=2 max=2) [if] [in] /* Edge list + conditions.	*/
    , Measure(string) /* Network measure to compute, e.g. distance.	*/
    [
    Name(string) /* Name of mata matrix to store returned object.	*/
    Weight_(varname numeric) /* Numeric edge weight variable.		*/
    /* Underscore attached since Stata confuses macros with var weights.*/
    Beta(varlist numeric min=2 max=2) /* Beta vector for Katz-Bonacich.	*/
    Label(string) /* Root name for returning vertex label.		*/
    DIrected /* Specified for directed networks.  Default is undirected.*/
    noDots /* Suppress status dots.					*/
    Infinity(real -101) /* Replacement for infinite distances.		*/
    Power /* Indicator for using power iteration.			*/
    noTranspose /* No transpose during eig. and KB cent. calculations.	*/
    Alpha(real 1) /* Alpha parameter for Katz-Bonacich centrality.	*/	
    ] 
; #d cr
/************************************************************************/
/*****		PARSE SYNTAX AND ERROR HANDLE			    *****/
/************************************************************************/
/***** Error handle type consistency of input varlist.			*/
capture confirm numeric variable `varlist'
if _rc!=0{
  capture confirm string variable `varlist'
  if _rc!=0{
    di as error "varlist must be either both numeric or both string"
    exit 7
  }
  /* Set varlist type indicator to 'string.'				*/
  else local vartype string
}
/* Set varlist type indicator to 'numeric.'				*/
else local vartype numeric
/***** Error handle network measure. 				   	*/
capture _measure_parse , `measure'
if _rc!=0{
  di as error "network measure `measure' not allowed"
  exit 198
}
local measure `s(measure)'
/***** Error handle mata matrix name if specified.			*/
if "`name'"!=""{
  _name_parse `name'
  local name `s(name)'
}
else local name ""
/***** Error handle label() option if specified.			*/
if "`label'"!=""{
  _label_parse `label'
  local label `s(label)'
  capture drop `label'_source 
  capture drop `label'_target
  qui gen int `label'_source=.
  qui gen int `label'_target=.  
}
else local label ""
/************************************************************************/
/***** Error handle and adjust 'touse' temporary sample marker variable.*/
/************************************************************************/
/* Mark sample.  By default, observations with any missing 		*/
/* numeric and/or string values are excluded. 				*/
marksample touse, strok
/* Tokenize varlist.							*/
tokenize `varlist'
/***** Error handle parallel edges.					*/
qui duplicates report `1' `2' if `touse'
if r(unique_value)!=r(N){
  di as error "parallel edges not allowed (duplicate edges detected)"
  exit 198
}
/***** Error handle self-loops.						*/
qui count if `1'==`2' & `touse'
if r(N)!=0{
  di as error "self-loops not allowed"
  exit 198
}  
/***** Error handle non-positive weights.  As of 04/25/2011, none of SGL*/
/* routines can handle zero and/or negative weights.  May change later.	*/
if "`weight_'"!=""{
  qui sum `weight_'
  if r(min)<=0{
    mata: st_rclear()
    di "option weight() incorrectly specified"
    di "non-positive weight detected"
    exit 198
  }
}
/* Set 'touse' to zero if weight var specified and any entries missing.	*/
if "`weight_'"!=""{
  qui replace `touse'=0 if `touse' & `weight_'==.
}
/* Set 'touse' to zero if beta vector specified and any entries missing.*/
if "`beta'"!=""{
  foreach var in `beta' {
    qui replace `touse'=0 if `touse' & `var'==.
  }
}   
/************************************************************************/
/***** 			OBTAIN RESULTS			            *****/
/************************************************************************/
/***** Declare temporary results matrix and obtain results from main().	*/
tempname res rc
mata: `res'=.
mata: `rc'=main(`res')
mata: st_local("_rc",strofreal(`rc'))
/***** Exit if internal error occurs in main().				*/
if `_rc'!=0{
  /* Clear any save values.						*/
  mata st_eclear(); st_sclear(); st_rclear()
  /* Harmonize Mata and Stata error codes if necessary.			*/
  if `_rc'==3360 error(430)
  else error(`_rc')
}
/* Copy output matrix to user-specified matrix name if provided.	*/
if "`name'"!=""{
  mata `name'=`res'
  di "matrix `name' saved in Mata"
}    
/* Repose esample.							*/
ereturn repost, esample(`touse')
/***** Drop temporary mata objects.					*/
mata mata drop `res'
end

/************************************************************************/
/***** 			MATA SUBROUTINE			            *****/
/************************************************************************/
/* Initialize Mata environment.						*/
mata:
/* Set variable declarations on.					*/
mata set matastrict on
mata set matalnum off
/* Declare function.							*/
real scalar main(real matrix res){
  /* Declare local variables.						*/
  transmorphic matrix V /* Varlist matrix.				*/       
  class network_measure scalar mymeasure /* Class instance.		*/
  real matrix E /* Edge list with edge weights.				*/
  real matrix B /* 'touse' beta vector matrix.				*/
  real vector b /* 'touse' beta vector.					*/
  real vector w /* 'touse' edge weight vector.				*/  
  real vector o /* Unique originial ID list.				*/ 
  real scalar k /* Matrix T row counter.				*/
  real scalar i /* Row counter.						*/  
  real scalar rc /* Return code. 					*/
  string scalar g /* General string.					*/
  /**********************************************************************/  
  /***** Preparation for generating edge list.				*/
  /**********************************************************************/
  /***** Transfer varlist into Mata matrix.				*/
  if(st_local("vartype")=="numeric"){
    V=st_data(.,st_local("varlist"),st_local("touse"))
  }
  else V=st_sdata(.,st_local("varlist"),st_local("touse"))
  /***** Transfer weight vector into Mata vector if specified.		*/
  if(st_local("weight_")!=""){
    w=st_data(.,st_local("weight_"),st_local("touse"))
  }
  /* If no weight variable specified, define vector with all ones.	*/
  else w=J(rows(V),1,1)
  /***** Transfer beta vector matrix into Mata matrix if specified.	*/
  if(st_local("beta")!=""){
    B=st_data(.,st_local("beta"),st_local("touse"))
  }
  /**********************************************************************/
  /***** Generate edge list and make adjustments.			*/
  /**********************************************************************/
  /***** Value-label mapping.						*/
  /* Convert original IDs to string if numeric.				*/
  if(!isstring(V)) V=strofreal(V,"%200.0g")  
  /* Obtain a list of unique original IDs.				*/
  o=uniqrows((V[.,1]\V[.,2]))
  /* Create value-label.						*/
  st_vlmodify("value_label",(1::rows(o)),o)
  /***** Generate edge list.						*/
  E=J(rows(V),3,.)
  E[.,(1..2)]=st_vlsearch("value_label",V)
  /* Concatenate edge weights to edge list.				*/
  E[.,3]=w  
  /***** Generate beta vector if specified.				*/
  if(st_local("beta")!=""){
    b=uniqrows(((E[.,1]\E[.,2]),(B[.,1]\B[.,2])))[.,2]
    /* If betas are not uniquely defined for each vertex, exit w/ error.*/
    if(rows(b)!=rows(o)) return(503)
  }
  /***** Generate label variables if requested by user.			*/
  /* Note: 'label'_source and 'label'_target has already been checked   */
  /* for existence.  Also, new empty variables have been generated.	*/  
  if(st_local("label")!=""){
    /***** Return source and target labels from edge list.		*/
    st_store(.,(st_local("label")+"_source",st_local("label")+"_target")
      ,st_local("touse"),E[.,(1..2)])
  }    
  /***** Adjust edge list if network undirected.			*/
  /***** CAUTION: EDGE LIST IS ALTERED.					*/
  if(st_local("directed")==""){
    E=uniqrows((((E[.,1]\E[.,2]),(E[.,2]\E[.,1])),(E[.,3]\E[.,3])))
  }     
  /**********************************************************************/
  /***** Obtain network measure.					*/
  /**********************************************************************/   
  /* Generate instance of class wrapper for SGL.			*/
  mymeasure=network_measure()
  /* Specifify class member variables.					*/
  mymeasure.E=E
  mymeasure.nodots=st_local("dots")
  mymeasure.infinity=strtoreal(st_local("infinity"))
  mymeasure.power=st_local("power")
  mymeasure.notranspose=st_local("transpose")
  mymeasure.alpha=strtoreal(st_local("alpha"))
  if(st_local("beta")!="") mymeasure.beta=b
  /***** Return parameters in e().					*/
  g="ereturn post"
  stata(g)
  st_numscalar("e(vertices)", max(E[.,(1..2)]))
  st_numscalar("e(edges)", rows(E))
  if(st_local("directed")=="") st_global("e(network)","undirected")
  else st_global("e(network)","directed")
  if(st_local("weight_")=="") st_global("e(edge)","unweighted")
  else st_global("e(edge)","weighted")
  st_global("e(measure)",st_local("measure"))
  st_global("e(target)",tokens(st_local("varlist"))[2])
  st_global("e(source)",tokens(st_local("varlist"))[1])  
  st_global("e(cmdline)","network "+st_local("0"))  
  st_global("e(cmd)","network")
  /***** Set return code to zero.					*/
  rc=0
  /***** Obtain results.						*/
  if(st_local("measure")=="adjacency") 	  rc=mymeasure.adjacency()  
  if(st_local("measure")=="distance") 	  rc=mymeasure.distance()
  if(st_local("measure")=="path") 	  rc=mymeasure.path()
  if(st_local("measure")=="betweenness")  rc=mymeasure.betweenness()
  if(st_local("measure")=="clustering")   rc=mymeasure.clustering()
  if(st_local("measure")=="eigenvector")  rc=mymeasure.eigenvector()
  if(st_local("measure")=="maxalpha") 	  rc=mymeasure.maxalpha()
  if(st_local("measure")=="katzbonacich") rc=mymeasure.katzbonacich()
  /***** Obtain results matrix from class wrapper.			*/
  res=mymeasure.res  
  /* Return code - may be zero or nonzero.				*/
  return(rc)
}
end

/************************************************************************/
/* Subroutine to parse measure() required option.			*/
/************************************************************************/
program define _measure_parse, sclass
  #d;
  syntax [anything] 
  [, 
  Adjacency 
  Distance 
  Path 
  Betweenness 
  Clustering
  Eigenvector 
  Maxalpha 
  Katzbonacich
  ]
  ; #d cr
  /* Return specified measure.						*/
  if "`adjacency'"!=""{
    sreturn local measure `"`adjacency'"'
  }
  else if "`distance'"!=""{
    sreturn local measure `"`distance'"'
  }
  else if "`path'"!=""{
    sreturn local measure `"`path'"'
  }
  else if "`betweenness'"!=""{
    sreturn local measure `"`betweenness'"'
  }
  else if "`clustering'"!=""{
    sreturn local measure `"`clustering'"'
  }
  else if "`eigenvector'"!=""{
    sreturn local measure `"`eigenvector'"'
  }  
  else if "`maxalpha'"!=""{
    sreturn local measure `"`maxalpha'"'
  }
  else if "`katzbonacich'"!=""{
    sreturn local measure `"`katzbonacich'"'
  }    
end
/************************************************************************/
/* Subroutine to parse name() option.					*/
/************************************************************************/
program define _name_parse, sclass
  syntax anything [, replace]  
  capture mata sizeof(`anything')
  if _rc==0 & "`replace'"==""{
    di as error "mata matrix `anything' already defined"
    exit(110)
  }  
  sreturn local name `"`anything'"'
end
/************************************************************************/
/* Subroutine to parse label() option.					*/
/************************************************************************/
program define _label_parse, sclass
  syntax name(name=label) [, replace]  
  /* Check to see if variables already exist.				*/
  capture confirm new var `label'_source `label'_target
  /* If already in existence and if 'replace' option not specified, 	*/
  /* allow 'confirm' command to emit error and exit.			*/
  if _rc!=0 & "`replace'"==""{
    confirm new var `label'_source `label'_target
  }  
  /* Return prefix string.						*/
  sreturn local label `"`label'"'
end



